﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Persistence.Services;
using System.Collections.Generic;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.Charting;

namespace SilverlightApplication1
{
    public class ChartCustomPropertyProvider : ICustomPropertyProvider
    {

        #region ICustomPropertyProvider Members

        public CustomPropertyInfo[] GetCustomProperties()
        {
            // Create three custom properties to persist the Columns, Sorting and Group descriptors using proxy objects
            return new CustomPropertyInfo[]
            {
                new CustomPropertyInfo("SeriesMappings", typeof(List<SeriesProxy>)),
                new CustomPropertyInfo("SortDescriptors", typeof(List<SortDescriptorProxy>)),
                new CustomPropertyInfo("FilterDescriptors", typeof(List<FilterDescriptorProxy>))
            };
        }

        public void InitializeObject(object context)
        {
            if (context is RadChart)
            {
                RadChart chart = context as RadChart;
                chart.SortDescriptors.Clear();
                chart.FilterDescriptors.Clear();
                chart.SeriesMappings.Clear();
            }
        }

        public object InitializeValue(CustomPropertyInfo customPropertyInfo, object context)
        {
            return null;
        }

        public object ProvideValue(CustomPropertyInfo customPropertyInfo, object context)
        {
            RadChart chart = context as RadChart;
            if (customPropertyInfo.Name == "SeriesMappings")
            {
                // create proxies for all of the columns and save only specific properties
                List<SeriesProxy> proxies = new List<SeriesProxy>();
                foreach (SeriesMapping seria in chart.SeriesMappings)
                {
                    SeriesProxy pr = new SeriesProxy()
                    {
                        ChartAreaName = seria.ChartAreaName,
                        CollectionIndex = seria.CollectionIndex,
                        LegendLabel = seria.LegendLabel,
                    };

                    foreach (ItemMapping im in seria.ItemMappings)
                    {
                        pr.ItemMappings.Add(im);
                    }

                    foreach (ChartSortDescriptor descriptor in seria.SortDescriptors)
                    {
                        pr.SortDescriptors.Add(new SortDescriptorProxy()
                        {
                            Member = descriptor.Member,
                            SortDirection = descriptor.SortDirection,
                        });
                    }
                    pr.GroupSettings = new GroupSetting
                    {
                        GroupFieldName = seria.GroupingSettings.StackGroupFieldName,
                        ShouldCreateSeriesForLastGroup = seria.GroupingSettings.ShouldCreateSeriesForLastGroup,
                        ShouldFlattenSeries = seria.GroupingSettings.ShouldFlattenSeries
                    };
                    foreach (ChartGroupDescriptor cgd in seria.GroupingSettings.GroupDescriptors)
                    {
                        GroupDescriptorProxy grProxy = new GroupDescriptorProxy() { Member = cgd.Member, Type = cgd.MemberType };
                        pr.GroupSettings.GroupDescriptors.Add(grProxy);
                    }
                    foreach (ChartFilterDescriptor cfd in seria.FilterDescriptors)
                    {
                        pr.FilterDescriptors.Add(new FilterDescriptorProxy
                        {
                            IsCaseSensitive = cfd.IsCaseSensitive,
                            Member = cfd.Member,
                            Operator = cfd.Operator,
                            Value = cfd.Value
                        });
                    }

                    proxies.Add(pr);
                }
                return proxies;
            }
            else if (customPropertyInfo.Name == "SortDescriptors")
            {
                // create proxies for all of the sort descriptors and save only specific properties
                List<SortDescriptorProxy> proxies = new List<SortDescriptorProxy>();
                foreach (ChartSortDescriptor descriptor in chart.SortDescriptors)
                {
                    proxies.Add(new SortDescriptorProxy()
                    {
                        Member = descriptor.Member,
                        SortDirection = descriptor.SortDirection,
                    });
                }
                return proxies;
            }
            else if (customPropertyInfo.Name == "FilterDescriptors")
            {
                // create proxies for all of the filter descriptors and save only specific properties
                List<FilterDescriptorProxy> proxies = new List<FilterDescriptorProxy>();
                foreach (ChartFilterDescriptor cfd in chart.FilterDescriptors)
                {
                    FilterDescriptorProxy proxy = new FilterDescriptorProxy();

                    if (cfd.Value != Telerik.Windows.Data.FilterDescriptor.UnsetValue)
                    {
                        proxy.Member = cfd.Member;
                        proxy.Operator = cfd.Operator;
                        proxy.Value = cfd.Value;
                        proxy.Type = cfd.MemberType;
                    }
                    proxies.Add(proxy);
                }
                return proxies;
            }
            return null;
        }

        public void RestoreValue(CustomPropertyInfo customPropertyInfo, object context, object value)
        {
            RadChart chart = context as RadChart;
            if (customPropertyInfo.Name == "SeriesMappings")
            {
                List<SeriesProxy> savedProxies = value as List<SeriesProxy>;

                // restore properties on the saved columns:
                foreach (SeriesProxy proxy in savedProxies)
                {
                    SeriesMapping mapping = new SeriesMapping()
                    {
                        ChartAreaName = proxy.ChartAreaName,
                        CollectionIndex = proxy.CollectionIndex,
                        LegendLabel = proxy.LegendLabel
                    };

                    foreach (ItemMapping im in proxy.ItemMappings)
                    {
                        mapping.ItemMappings.Add(im);
                    }

                    foreach (FilterDescriptorProxy fd in proxy.FilterDescriptors)
                    {
                        ChartFilterDescriptor cfd = new ChartFilterDescriptor
                        {
                            Value = fd.Value,
                            Operator = fd.Operator,
                            MemberType = fd.Type,
                            Member = fd.Member,
                            IsCaseSensitive = fd.IsCaseSensitive
                        };
                        mapping.FilterDescriptors.Add(cfd);
                    }
                    GroupingSettings settings = new GroupingSettings
                    {
                        StackGroupFieldName = proxy.GroupSettings.GroupFieldName,
                        ShouldFlattenSeries = proxy.GroupSettings.ShouldFlattenSeries,
                        ShouldCreateSeriesForLastGroup = proxy.GroupSettings.ShouldCreateSeriesForLastGroup
                    };
                    foreach (GroupDescriptorProxy gdp in proxy.GroupSettings.GroupDescriptors)
                    {
                        settings.GroupDescriptors.Add(new ChartGroupDescriptor
                        {
                            MemberType = gdp.Type,
                            Member = gdp.Member
                        });
                    }
                    mapping.GroupingSettings = settings;
                    foreach (SortDescriptorProxy sdp in proxy.SortDescriptors)
                    {
                        ChartSortDescriptor csd = new ChartSortDescriptor { Member = sdp.Member, SortDirection = sdp.SortDirection };
                        mapping.SortDescriptors.Add(csd);
                    }

                    chart.SeriesMappings.Add(mapping);
                }
            }
            else if (customPropertyInfo.Name == "SortDescriptors")
            {
                chart.SortDescriptors.Clear();
                List<SortDescriptorProxy> savedProxies = value as List<SortDescriptorProxy>;

                // restore properties on the saved sort descriptors:
                foreach (SortDescriptorProxy proxy in savedProxies)
                {
                    chart.SortDescriptors.Add(new ChartSortDescriptor() { Member = proxy.Member, SortDirection = proxy.SortDirection });
                }
            }
            else if (customPropertyInfo.Name == "FilterDescriptors")
            {
                chart.FilterDescriptors.Clear();
                List<FilterDescriptorProxy> savedProxies = value as List<FilterDescriptorProxy>;
                foreach (FilterDescriptorProxy proxy in savedProxies)
                {
                    chart.FilterDescriptors.Add(new ChartFilterDescriptor()
                    {
                        Value = proxy.Value,
                        Operator = proxy.Operator,
                        Member = proxy.Member,
                        IsCaseSensitive = proxy.IsCaseSensitive
                    });
                }
            }
        }

        private SeriesMapping GetSeriesMapping(RadChart chart, string label)
        {
            foreach (SeriesMapping sm in chart.SeriesMappings)
            {
                if (sm.LegendLabel == label)
                    return sm;
            }
            return null;
        }

        #endregion
    }
}
